(*
 * Graph: generic graph library
 * Copyright (C) 2004
 * Sylvain Conchon, Jean-Christophe Filliatre and Julien Signoles
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(**  Goldberg maximal flow algorithm *)

(** Minimal graph signature for Goldberg *)
module type G = sig
  type t
  module V : Sig.COMPARABLE
  module E : sig 
    type t 
    type label 
    val compare : t -> t -> int
    val create : V.t -> label -> V.t -> t
    val label : t -> label 
    val dst : t -> V.t 
    val src : t -> V.t
  end 
  val nb_vertex : t -> int
  val iter_vertex : (V.t -> unit) -> t -> unit
  val iter_edges : (V.t -> V.t -> unit) -> t -> unit
  val fold_pred : (V.t -> 'a -> 'a) -> t -> V.t -> 'a -> 'a
  val fold_succ_e : (E.t -> 'a -> 'a) -> t -> V.t -> 'a -> 'a
  val fold_pred_e : (E.t -> 'a -> 'a) -> t -> V.t -> 'a -> 'a
end

(** Signature for edges' flow *)
module type FLOW = sig
  type label
  type t
  val flow : label -> t
  val add : t -> t -> t
  val sub : t -> t -> t
  val zero : t
  val compare : t -> t -> int
  val min : t -> t -> t
end

module Make(G: G)(F: FLOW with type label = G.E.label) : 
sig
      
  val maxflow : G.t -> G.V.t -> G.V.t -> (G.V.t -> G.V.t -> F.t) * F.t
    (** [maxflow g v1 v2] searchs the maximal flow from [v1] to [v2]
      using the Goldberg algorithm. It returns the new flows on each
      edges and the growth of the flow. *)

end
